import { Observable } from 'rxjs'

export default class SgEvent {
  constructor (object) {
    this.listeners = {}
    this.object = object
  }

  /**
   * kelsey yau
   * 添加事件绑定
   * @param {*} eventObj object 事件对象，键：string 事件类型，值：function 回调函数，如：{'click': listener}
   */
  on (eventObj) {
    for (let type in eventObj) {
      if (Object.prototype.hasOwnProperty.call(eventObj, type)) {
        const subscription = this._eventProcess(type, eventObj[type])
        this.registerFunc(type, eventObj[type], subscription)
      }
    }
  }

  /**
   * kelsey yau
   * 事件注册
   * @param {*} type string 事件类型
   * @param {*} func function 事件回调函数
   * @param {*} subscription object 事件订阅
   */
  registerFunc (type, func, subscription) {
    const eventObj = { callback: func, observer: subscription }
    if (Object.prototype.hasOwnProperty.call(this.listeners, type)) {
      if (this.listeners[type]) {
        this.listeners[type].push(eventObj)
      } else {
        let funcArray = []
        funcArray.push(eventObj)
        this.listeners[type] = funcArray
      }
    } else {
      let funArry = []
      funArry.push(eventObj)
      this.listeners[type] = funArry
    }
  }

  /**
   * kelsey yau
   * 事件处理器
   * @param {*} type string 事件类型
   * @param {*} func function 回调函数
   */
  _eventProcess (type, func) {
    const observable = new Observable((subscriber) => {
      this.object.on(type, (event) => {
        subscriber.next(event)
      })
    })
    const subscription = observable.subscribe((event) => {
      func(event)
    })
    return subscription
  }

  /**
   * kelsey yau
   * 取消注册
   * @param {*} type string 事件类型
   * @param {*} func function 回调函数
   */
  unregisterFunc (type, func) {
    let funArry = this.listeners[type]
    if (funArry && funArry.length > 0) {
      let index = funArry.findIndex((value) => {
        return value['callback'] === func
      })
      funArry[index]['observer'].unsubscribe()
      funArry.splice(index, 1)
    }
  }

  /**
   * kelsey yau
   * 事件解绑
   * @param {*} eventObj object 事件对象，键：string 事件类型，值：function 回调函数，如：{'click': listener}
   */
  un (eventObj) {
    for (let type in eventObj) {
      if (Object.prototype.hasOwnProperty.call(eventObj, type)) {
        this.unregisterFunc(type, eventObj[type])
      }
    }
  }

  /**
   * kelsey yau
   * 销毁所有事件
   */
  destroy () {
    for (let type in this.listeners) {
      let funArry = this.listeners[type]
      if (funArry) {
        funArry.forEach((eventObj) => {
          eventObj.observer.unsubscribe()
        })
      }
    }
    this.listeners = {}
    this.object = null
  }
}
