import { loadModules } from 'esri-loader'
import MAP_OPTIONS from '../../config/baseConfig'

export default class SgLayerFactory {
  /**
   * kelsey yau
   * 创建并加载图层至地图中
   * @param {*} config 图层配置
   * @param {*} sgMap 地图对象
   */
  static addConfigLayer (config, sgMap) {
    let baseLayerConfig = {
      id: config.id,
      url: config.url,
      layerName: config.layerName,
      visible: config.visible,
      opacity: config.opacity ? config.opacity : 1
    }

    switch (config.type) {
      case 'UnKnowLayer':
        return null
      case 'ArcGISTiledMapServiceLayer':
        loadModules(['esri/layers/TileLayer'], MAP_OPTIONS).then(([TileLayer]) => {
          const tileLayer = new TileLayer(baseLayerConfig)
          sgMap.map.add(tileLayer)
        })
        break
      case 'ArcGISDynamicMapServiceLayer':
        loadModules(['esri/layers/MapImageLayer'], MAP_OPTIONS).then(([MapImageLayer]) => {
          const mapImageLayer = new MapImageLayer(baseLayerConfig)
          sgMap.map.add(mapImageLayer)
        })
        break
      case 'ArcGISImageMapServiceLayer':
        loadModules(['esri/layers/ImageryLayer'], MAP_OPTIONS).then(([ImageryLayer]) => {
          const imageryLayer = new ImageryLayer(baseLayerConfig)
          sgMap.map.add(imageryLayer)
        })
        break
      case 'ArcGISFeatureMapServiceLayer':
        loadModules(['esri/layers/FeatureLayer'], MAP_OPTIONS).then(([FeatureLayer]) => {
          const featureLayer = new FeatureLayer(baseLayerConfig)
          sgMap.map.add(featureLayer)
        })
        break
      case 'WmsServiceLayer':
        loadModules(['esri/layers/WMSLayer'], MAP_OPTIONS).then(([WMSLayer]) => {
          baseLayerConfig['sublayers'] = config.sublayers ? config.sublayers : null
          const wmsLayer = new WMSLayer(baseLayerConfig)
          sgMap.map.add(wmsLayer)
        })
        break
      case 'WmtsServiceLayer':
        loadModules(['esri/layers/WMTSLayer'], MAP_OPTIONS).then(([WMTSLayer]) => {
          baseLayerConfig['activeLayer'] = config.activeLayer ? config.activeLayer : null
          const wmtsLayer = new WMTSLayer(baseLayerConfig)
          sgMap.map.add(wmtsLayer)
        })
        break
      case 'WebTileLayer':
        loadModules(['esri/layers/WebTileLayer'], MAP_OPTIONS).then(([WebTileLayer]) => {
          baseLayerConfig['subDomains'] = config.subDomains ? config.subDomains : null
          const webTileLayer = new WebTileLayer(baseLayerConfig)
          sgMap.map.add(webTileLayer)
        })
        break
      case 'VectorTileLayer':
        loadModules(['esri/layers/VectorTileLayer'], MAP_OPTIONS).then(([VectorTileLayer]) => {
          baseLayerConfig['style'] = config.style ? config.style : null
          const vectorTileLayer = new VectorTileLayer(baseLayerConfig)
          sgMap.map.add(vectorTileLayer)
        })
        break
      default:
        break
    }
  }
}
