import SgOperation from './SgOperation'
import SgViewManager from '../manage/view/SgViewManager'
import SgHighlightOperation from './SgHighlightOperation'
import SgQueryOperation from './SgQueryOperation'
import { loadModules } from 'esri-loader'
import MAP_OPTIONS from '../config/baseConfig'
import SgMapOverlayer from '../overlayer/SgMapOverlayer'

export default class SgSelectOperation extends SgOperation {
  constructor (sgMap) {
    super(sgMap)
    this.viewManager = SgViewManager.getInstance(sgMap)
    this.highLightOperation = SgHighlightOperation.getInstance(sgMap)
    this.queryOperation = new SgQueryOperation(sgMap)
    this.geometryes = []
    this.sgOverlay = SgMapOverlayer.getInstance(sgMap)
    loadModules(['esri/geometry/geometryEngine'], MAP_OPTIONS).then(([geometryEngine]) => {
      this.geometryEngine = geometryEngine
    })
    this.selectListener = (event) => {
      this.geometryes = []
      if (event.button === 0) {
        this._getFeatureAndHighlight(event)
      } else {
        this.unSelect()
      }
    }
    this.selectMoveListener = (event) => {
      this.sgOverlay.updateInfoWindow('select', { x: event.x, y: event.y }, 'screen')
    }
  }

  /**
   * 创建选择
   */
  createMapFeatureSelect () {
    this.sgOverlay.addInfoWindow(
      { x: 0, y: 0, spatialReference: this.map.sr, type: 'point' },
      { id: 'select', content: '左键点击选择<br/>右键点击结束' })
    this.viewManager.changeMapContainerCursor('crosshair')
    this.viewManager.on({
      'click': this.selectListener,
      'pointer-move': this.selectMoveListener
    })
  }

  /**
   * 获取数据并高亮显示，私有方法
   * @param {*} point 坐标数据
   */
  _getFeatureAndHighlight (point) {
    this.viewManager.getFeaturesAtScreenPoint(point).then(features => {
      if (features.results.length > 0) {
        features.results.forEach((item) => {
          this.geometryes.push(item.graphic.geometry)
        })
      }
      this._getIentifyFeature(point.mapPoint)
      this.queryOperation.onMapIdentify((result) => {
        const identifyData = result.results
        if (identifyData.length > 0) {
          identifyData.forEach((graphic) => {
            let equal = this.geometryes.some(geom => {
              return this.geometryEngine.equals(geom, graphic.feature.geometry)
            })
            if (!equal) {
              this.geometryes.push(graphic.feature.geometry)
            }
          })
        }
        this.highLightOperation.setHighLight(this.geometryes, false)
      })
    })
  }

  /**
   * 获取identify查询的要素
   * @param {*} point 坐标点
   */
  _getIentifyFeature (point) {
    this.queryOperation._activeMapIdentify(null, point)
  }

  /**
   * 取消选择
   */
  unSelect () {
    this.viewManager.un({
      'click': this.selectListener,
      'pointer-move': this.selectMoveListener
    })
    this.highLightOperation.clearHighlightLayer()
    this.sgOverlay.clearInfoWindowById('select')
    this.viewManager.changeMapContainerCursor('auto')
  }
}
