import { styleConfig } from '../config/styleConfig'

export function createGraphic (sgMap, geometry, symbol) {
  const graphic = sgMap.graphic.clone()
  graphic.geometry = geometry
  if (symbol) {
    graphic.symbol = symbol
  } else {
    switch (geometry.type) {
      case 'point':
        graphic.symbol = styleConfig.DEFAULT_POINT
        break
      case 'polyline':
        graphic.symbol = styleConfig.DEFAULT_LINE
        break
      case 'polygon':
        graphic.symbol = styleConfig.DEFAULT_POLYGON
        break
      default:
        break
    }
  }
  return graphic
}

export function createGeometry (sgMap, option) {
  switch (option.type) {
    case 'point':
      if (option instanceof sgMap.Point) {
        return option
      } else {
        return new sgMap.Point(option)
      }
    case 'polyline':
      if (option instanceof sgMap.Polyline) {
        return option
      } else {
        return new sgMap.Polyline(option)
      }
    case 'polygon':
      if (option instanceof sgMap.Polygon) {
        return option
      } else {
        return new sgMap.Polygon(option)
      }
    default:
      return null
  }
}

/**
 * 根据点坐标生成点要素
 * @param {*} sgMap sgMap对象实例
 * @param {*} point 点坐标 如 {x: 213232, y: 23446457}
 * @param {*} symbol 点样式
 */
export function createPointGraphic (sgMap, point, symbol) {
  let pointGeom = {
    type: 'point',
    x: point.x,
    y: point.y,
    spatialReference: sgMap.sr
  }
  const graphic = sgMap.graphic.clone()
  graphic.geometry = pointGeom
  graphic.symbol = symbol
  return graphic
}

/**
 * 根据坐标点集创建线要素
 * @param {*} sgMap sgMap对象实例
 * @param {*} paths 坐标点集
 * @param {*} symbol 线样式
 */
export function createLineGraphic (sgMap, paths, symbol) {
  let polyline = {
    type: 'polyline',
    paths: paths,
    spatialReference: sgMap.sr
  }
  const graphic = sgMap.graphic.clone()
  graphic.geometry = polyline
  graphic.symbol = symbol
  return graphic
}

export function createPolygonGraphic (sgMap, rings, symbol) {
  let polygon = {
    type: 'polygon',
    rings: rings,
    spatialReference: sgMap.sr
  }
  const graphic = sgMap.graphic.clone()
  graphic.geometry = polygon
  graphic.symbol = symbol
  return graphic
}

export function createCircleGraphic (sgMap, centerPoint, radius, symbol) {
  let circle = new sgMap.Circle({
    hasZ: false,
    hasM: false,
    center: centerPoint,
    radius: radius,
    spatialReference: sgMap.sr
  })
  const graphic = sgMap.graphic.clone()
  graphic.geometry = circle
  graphic.symbol = symbol
  return graphic
}

export function createRetangleGraphic (sgMap, coordinates, symbol) {
  if (coordinates.length < 2) {
    return
  }
  let rings = [[
    coordinates[0],
    [coordinates[0][0], coordinates[1][1]],
    coordinates[1],
    [coordinates[1][0], coordinates[0][1]]
  ]]
  let graphic = createPolygonGraphic(sgMap, rings, symbol)
  return graphic
}
